/*
 *  Copyright (C) 2001 Philip Langdale
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "galeon.h"
#include "history.h" 
#include "mozilla.h"

#include "nsCOMPtr.h"
#include "nsISupportsArray.h"
#include "nsIServiceManager.h"
#include "nsIRequestObserver.h"
#include "nsReadableUtils.h"
#include "nsIURI.h"

#include "GlobalHistory.h"

#if MOZILLA_SNAPSHOT > 13
NS_IMPL_ISUPPORTS2(GlobalHistory, nsIGlobalHistory, nsIGlobalHistory2)
#else
NS_IMPL_ISUPPORTS2(GlobalHistory, nsIGlobalHistory, nsIBrowserHistory)
#endif

GlobalHistory::GlobalHistory ()
{
	NS_INIT_ISUPPORTS();
}

GlobalHistory::~GlobalHistory ()
{
}

/* void addPage (in string aURL); */
#if MOZILLA_SNAPSHOT > 13
NS_IMETHODIMP GlobalHistory::AddURI(nsIURI *aURI,
				    PRBool aRedirect,
				    PRBool aTopLevel)
{
	nsCAutoString spec;
	aURI->GetSpec(spec);

	mLastPageVisited.Assign(spec);

	history_visited(spec.get());

	return NS_OK;
}
#endif

NS_IMETHODIMP GlobalHistory::AddPage (const char *aURL)
{
	history_visited (aURL);
	return NS_OK;
}

/* boolean isVisited (in string aURL); */
#if MOZILLA_SNAPSHOT > 13
NS_IMETHODIMP GlobalHistory::IsVisited(nsIURI *aURI, PRBool *_retval)
{
	nsCAutoString spec;
	aURI->GetSpec(spec);

	*_retval = history_is_visited(spec.get());

	return NS_OK;
}
#endif

NS_IMETHODIMP GlobalHistory::IsVisited (const char *aURL, PRBool *_retval)
{
	*_retval = history_is_visited (aURL);
	return NS_OK;
}

/* void setPageTitle (in string aURL, in wstring aTitle); */
#if MOZILLA_SNAPSHOT > 13
NS_IMETHODIMP GlobalHistory::SetPageTitle(nsIURI *aURI,
					  const nsAString& aTitle)
{
	/* check for NULL title (it happens, strangely) */
	if (aTitle.IsEmpty())
	{
		/* not sure what we should do, so... */
		return NS_OK;
	}

	nsCAutoString spec;
	aURI->GetSpec(spec);

	gchar *title_locale;
	gchar *title_utf8;

	/* attempt conversion */
	title_locale = mozilla_unicode_to_locale(PromiseFlatString(aTitle).get());
	title_utf8 = mozilla_unicode_to_utf8(PromiseFlatString(aTitle).get());
	
	/* set page title */
	history_set_page_title (spec.get(), title_locale, title_utf8);

	/* NB: we don't free the strings since set_page_title
	 * will hold onto them -- this is just to save some 
	 * unnecessary g_strdup()ing -- Matt */

	/* done */
	return NS_OK;
}
#else
NS_IMETHODIMP GlobalHistory::SetPageTitle (const char *aURL, 
					   const PRUnichar *aTitle)
{
	gchar *title_locale;
	gchar *title_utf8;

	/* check for NULL title (it happens, strangely) */
	if (aTitle == NULL)
	{
		/* not sure what we should do, so... */
		return NS_OK;
	}

	/* attempt conversion */
	title_locale = mozilla_unicode_to_locale (aTitle);
	title_utf8 = mozilla_unicode_to_utf8 (aTitle);
	
	/* set page title */
	history_set_page_title (aURL, title_locale, title_utf8);

	/* NB: we don't free the strings since set_page_title
	 * will hold onto them -- this is just to save some 
	 * unnecessary g_strdup()ing -- Matt */

	/* done */
	return NS_OK;
}
#endif

/* void removePage (in string aURL); */
NS_IMETHODIMP GlobalHistory::RemovePage(const char *aURL)
{
	return NS_ERROR_NOT_IMPLEMENTED;
}

/* void removePagesFromHost (in string aHost, in boolean aEntireDomain); */
NS_IMETHODIMP GlobalHistory::RemovePagesFromHost(const char *aHost, 
						 PRBool aEntireDomain)
{
	return NS_ERROR_NOT_IMPLEMENTED;
}

/* void removeAllPages (); */
NS_IMETHODIMP GlobalHistory::RemoveAllPages()
{
	return NS_ERROR_NOT_IMPLEMENTED;
}

#if MOZILLA_SNAPSHOT > 14
/* readonly attribute string lastPageVisited; */
NS_IMETHODIMP GlobalHistory::GetLastPageVisited(nsACString& aLastPageVisited)
{
	aLastPageVisited.Assign(mLastPageVisited);
	return NS_OK;
}
#else
/* readonly attribute string lastPageVisited; */
NS_IMETHODIMP GlobalHistory::GetLastPageVisited(char **aLastPageVisited)
{
	*aLastPageVisited = ToNewCString(mLastPageVisited);
	return NS_OK;
}
#endif

#if MOZILLA_SNAPSHOT > 8 && MOZILLA_SNAPSHOT <= 13
NS_IMETHODIMP GlobalHistory::SetLastPageVisited(const char * aLastPageVisited)
{
	mLastPageVisited.Assign(aLastPageVisited);
	return NS_OK;
}
#endif

#if MOZILLA_SNAPSHOT > 13
NS_IMETHODIMP GlobalHistory::HidePage(nsIURI *url)
{
	return NS_ERROR_NOT_IMPLEMENTED;

}
#else
NS_IMETHODIMP GlobalHistory::HidePage(const char *url)
{
	return NS_ERROR_NOT_IMPLEMENTED;

}
#endif

/* readonly attribute PRUint32 count; */
NS_IMETHODIMP GlobalHistory::GetCount(PRUint32 *aCount)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

#if MOZILLA_SNAPSHOT <= 8
/* void startBatchUpdate (); */
NS_IMETHODIMP GlobalHistory::StartBatchUpdate()
{
    return NS_ERROR_NOT_IMPLEMENTED;
}
  
/* void endBatchUpdate (); */
NS_IMETHODIMP GlobalHistory::EndBatchUpdate()
{
    return NS_ERROR_NOT_IMPLEMENTED;
}
#endif

/* void markPageAsTyped (in string url); */
NS_IMETHODIMP GlobalHistory::MarkPageAsTyped(const char *url)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

#if MOZILLA_SNAPSHOT == 5
/* Described by mozilla.org as a temporary ugly hack. We will never need to
 * implement it. It is here to allow compilation.
 */
/* void outputReferrerURL (in string aURL, in string aReferrer); */
NS_IMETHODIMP GlobalHistory::OutputReferrerURL(const char *aURL, const char *aReferrer)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}
#endif
